/* Driver for RATOC SMA03 SmartMedia reader
 * Header File
 *
 * sma03.h, v0.20
 *
 * Current development and maintenance by:
 *   (c) 2001 RATOC Systems, Inc.
 *
 * See sma03.c for more explanation
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _USB_SMA03_H
#define _USB_SMA03_H

/* SSFDC Command */
#define	SSFDC_WRDATA    0x80
#define	SSFDC_READ      0x00
#define	SSFCD_READ_SPR  0x50
#define	SSFDC_READ1     0x00
#define	SSFDC_READ2     0x01
#define	SSFDC_READ3     0x50
#define	SSFDC_RST_CHIP  0xFF
#define	SSFDC_WRITE     0x10
#define	SSFDC_ERASE1    0x60
#define	SSFDC_ERASE2    0xD0
#define	SSFDC_RDSTATUS  0x70
#define	SSFDC_READ_ID   0x90

/* SSFDC Status */
#define	WR_FAIL         0x01    /* 0:Pass,          1:Fail */
#define	SUSPENDED       0x20    /* 0:Not Suspended, 1:Suspended */
#define	READY           0x40    /* 0:Busy,          1:Ready */
#define	WR_PRTCT        0x80    /* 0:Protect,       1:Not Protect */

#define	SM_STATUS_FAIL		0x01	/* 0:Pass,          1:Fail */
#define	SM_STATUS_READY		0x40	/* 0:Busy,          1:Ready */
#define	SM_STATUS_WR_PROTECT	0x80	/* 0:Protect,       1:Not Protect */

/*
 * SMA03 MB89589 Port Map
 */
#define SMA03PORT_USB_DATA   0x00  /* [R/W] Data Reg. */
#define SMA03PORT_USB_STAT   0x01  /* [R/-] Status Reg. */
#define SMA03PORT_USB_MODE   0x02  /* [R/W] Mode Reg. */

/*
 * SMA03 MB89589 Control Status Bit
 */
#define SMA03_STAT_BUSY    0x80
#define SMA03_STAT_MODEL   0x40
#define SMA03_STAT_PWR     0x10
#define SMA03_STAT_STCHG   0x08
#define SMA03_STAT_CENB    0x04
#define SMA03_STAT_HWECC   0x02
#define SMA03_STAT_WPD     0x01

#define SMA03_STAT_INIT    (SMA03_STAT_BUSY | SMA03_STAT_STCHG)
/*
 * SMA03 MB89589 Mode Byte
 */
#define SMA03_MODE_STANDBY  0x00
#define SMA03_MODE_WR_DATA  0x90
#define SMA03_MODE_WR_CMD   0x91
#define SMA03_MODE_WR_ADR   0x92
#define SMA03_MODE_PW_OFF   0x98
#define SMA03_MODE_PW_ON    0x9C
/*
 * SMA03 Vendor-specific Request Codes
 */
#define SMA03_IN_DATA       0x08  /* Read from register in drirect */
#define SMA03_OUT_DATA      0x0A  /* Write to register in direct */
#define SMA03_READ_PAGE     0x84   /* Read page of SmartMedia */
#define SMA03_READ_REDU     0x85   /* Read redu fo SmartMedia */
#define SMA03_READ_ID       0x86   /* Read ID of SmartMedia */
#define SMA03_READ_STATUS   0x87   /* Read status of SmartMedia */
#define SMA03_REDU_BLOCK    0x89   /* Read all redu in block */
#define SMA03_READ_HIDDEN   0x8D   /* Read hidden block of SmartMedia */
#define SMA03_READ_BLOCK    0x94  /* Read any pages in block */
#define SMA03_ERASE_BLOCK   0xA3   /* Erase all page in block */
#define SMA03_WRITE_PAGE    0xA4   /* Write page fo SmartMedia */
#define SMA03_WRITE_REDU    0xA5   /* Write redu fo SmartMedia */
#define SMA03_WRITE_BLOCK    0xB4  /* Write any pages in block */
#define SMA03_READ_STATUS2  0xB7  /* Read status after WriteBlockCommand(B4) */
#define SMA03_GET_FW_INFO   0xC0   /* Get information of firmware */
#define SMA03_RESET_MEDIA   0xE0   /* Reset for SmartMedia */

/* bulk transport endpoint */
#define SMA03_DATA_WRITE    0x01   /* EP1 */
#define SMA03_DATA_READ     0x02   /* EP2 */
#define SMA03_CTRL_WRITE    0x03   /* EP3 */

/*
 * FW Ver2.03
 * GET_FW_INFO(0xc0)
 *
 * BmRequestType bRequest wValue wIndex wLength
 *     0xc0        0xc0     0      0       8
 */
typedef struct
{
    unsigned char   CodeLevel;
        // b7/b6 write function level
        //  00: WRITE BLOCK(0xB4)
        //  01: WRITE BLOCK(0xB4)
        // b5/b4 read function level
        //  00: READ BLOCK(0x94)
        //  01: READ BLOCK(0x94)
        // b3/b2 RESERVED
        // b1/b0 RESERVED
    unsigned char   StandardLevel;      // SSFDC Standard
        // 0x00: SSFDC Standard 1999
        // 0x01: SSFDC Standard 2000
    unsigned char   Version;
    unsigned char   Revision;
    unsigned char   FWType;
    unsigned char   Reserved[3];
} SMA03_FW_INFO, *PSMA03_FW_INFO;


#define		CLEVEL_WRITEBLOCK_SUPPORT	0x40
#define		CLEVEL_READBLOCK_SUPPORT	0x10


/*******************************
    Redundant Area Point
*******************************/
#define DSA	4
#define BSA	5
#define BAA10	6
#define BAA11	7
#define ECA20	8
#define ECA21	9
#define ECA22	10
#define BAA20	11
#define BAA21	12
#define ECA10	13
#define ECA11	14
#define ECA12	15
/*******************************
	ECC MASK BIT
*******************************/
#define BIT7    0x80
#define BIT6    0x40
#define BIT5    0x20
#define BIT4    0x10
#define BIT3    0x08
#define BIT2    0x04
#define BIT1    0x02
#define BIT0    0x01
#define BIT0011 0x03
#define BIT23   0x00800000L
#define CORRECTABLE   0x00555554L
/*******************************
      Counting Value
*******************************/
#define SMA_SECTOR 512	/* Sector Lengh  */
#define REDUD  16	/* Redundant Lengh  */
#define Page256M 256
#define Page512M 512
#define	MaxBlk  1000
#define	MAX_BLOCK_PER_ZONE_LOGICAL      1000
#define	MAX_BLOCK_PER_ZONE_PHYSICAL     1024
/*******************************
   ECC READ or WRITE MODE
*******************************/
#define READ_DIRECTION    0
#define WRITE_DIRECTION   1

/*
 *  structure of redundant area
 */
typedef struct
{
    unsigned char   Reserved[4];            /*       512-515 */
    unsigned char   DataStatusArea;        /* DSA : 516 */
    unsigned char   BlockStatusArea;       /* BSA : 517 */
    unsigned char   BlockAddressArea1[2];  /* BAA1 : 518-519 */
    unsigned char   EccArea2[3];           /* ECC  : 520-522 */
    unsigned char   BlockAddressArea2[2];  /* BAA2 : 523-524 */
    unsigned char   EccArea1[3];           /* ECC  : 525-527 */
} REDUNDANT_AREA, *PREDUNDANT_AREA;

typedef struct
{
    unsigned char   Reserved[4];           /*       256-259 */
    unsigned char   DataStatusArea;       /* DSA : 260 */
    unsigned char   BlockStatusArea;      /* BSA : 261 */
    unsigned char   BlockAddressArea1[2];  /* BAA1 : 262-263 */
} REDUNDANT_AREA256_0, *PREDUNDANT_AREA256_0;

typedef struct
{
    unsigned char   EccArea2[3];           /* ECC  : 256-258 */
    unsigned char   BlockAddressArea2[2];  /* BAA2 : 259-260 */
    unsigned char   EccArea1[3];           /* ECC  : 261-263 */
} REDUNDANT_AREA256_1, *PREDUNDANT_AREA256_1;

typedef struct
{
    unsigned char   DataArea[512];
    REDUNDANT_AREA  RedundantArea;
    unsigned char   Padding[48];
} BLOCKREAD_MEMORYMAP512, *PBLOCKREAD_MEMORYMAP512;  // Total 576byte

typedef struct
{
    unsigned char        DataArea0[256];
    REDUNDANT_AREA256_0  RedundantArea0;
    unsigned char        Padding0[56];
    unsigned char        DataArea1[256];
    REDUNDANT_AREA256_1  RedundantArea1;
    unsigned char        Padding1[56];
} BLOCKREAD_MEMORYMAP256, *PBLOCKREAD_MEMORYMAP256;  // Total 640byte

/* RATOC SMA03 stuff */

extern int sma03_transport(Scsi_Cmnd *srb, struct us_data *us);

union CD_ADDR {
    unsigned long  Adrs;         /* Card Address A08(9)-A20(21) */
    struct adr {
        unsigned char adr0;  /* Card Address A08(9)-A15(16) */
        unsigned char adr1;  /* Card Address A16(17)-A20(21) */
        unsigned char adr2;  /* Card Address A08(9)-A15(16) */
        unsigned char adr3;  /* Card Address A16(17)-A20(21) */
    } b;
};

typedef struct {
    unsigned char buf1[256];  /* Common Data Buffer */
    unsigned char rdt1[8];    /* Redundant Data Buffer */
    unsigned char buf2[256];  /* Common Data Buffer */
    unsigned char rdt2[8];    /* Redundant Data Buffer */
} SMEDIA;

typedef struct {
    unsigned char buf[512];   /* Common Data Buffer */
    unsigned char rdt[8];     /* Redundant Data Buffer */
} BMEDIA;

union MEDIABUF {
    SMEDIA SMedia;
    BMEDIA BMedia;
};

struct sma03_card_info {
    unsigned short ComputeGeomFlag;

    long    capacity;       /* Size of card in bytes */
    int     pagesize;       /* Size of page in bytes */
    int     redusize;       /* Redu of page in bytes */
    int     pageshift;      /* log2 of pagesize */
    int     blocksize;      /* Size of block in pages */
    int     SectorPerBlock;
    int     blockshift;     /* log2 of blocksize */
    int     blockmask;      /* 2^blockshift - 1 */
/*        int     zone;         * Zone in card */
    int     goodblocks;     /* Good blocks in zone */

    union CD_ADDR  Card;    /* Physical address of card */

    int (*read_sector)(struct us_data *,
                union CD_ADDR, unsigned char *, unsigned char *);
                            /* read function */
    int (*write_sector)(struct us_data *,
                union CD_ADDR, unsigned char *, unsigned char *);
                            /* write function */
    int (*read_block)(struct us_data *,
                PBLOCKREAD_MEMORYMAP512, unsigned short);
                            /* read block function */
    int (*write_block)(struct us_data *,
                unsigned short, unsigned short, short *);
                            /* write block function */
    int (*erase_block)(struct us_data *us,
                unsigned short DeletePhysicalBlock,
                unsigned short *pMediaStatus );
                            /* Erase block function */
    int (*check_ecc)(struct us_data *, unsigned char *, unsigned int);
                                /* read function */
    int     MaskROM;        /* 1: ROM  0:Flash */
    int     EccVerify;       /* 1: On  0: Off */
    int     WriteVerify;     /* 1: On  0: Off */
    int     ReadOnly;       /* ROM or Write-protected */

    unsigned short TotalLogicalBlock;   /* Allocation Table Total Block Count */
    unsigned short TotalPhysicalBlock;  /* SSFDC Total Block Count */
    unsigned short TotalZones;          /* SSFDC Total Zone Count */
    unsigned short PagePerSector;      /* 1page(512type) or 2page(256type) */
    unsigned short Redud;        /* Redundant Data Count */
    unsigned short BitShift;     /* Address Bit Shift Value */

    unsigned long  ac_cunt;      /* Sector no. demand */
    unsigned long  ac_strt;      /* Start Sector no. */
    unsigned short ac_zone;      /* Zone no. */
    unsigned short ac_block;     /* Bock no. */
    unsigned short secnm;        /* Access Sector no. in block. */
    unsigned short ac_page;      /* Access Page no. in block */
    unsigned char *trspt;        /* transfer buffer */

    unsigned short BadBlockCountPerZone[8];  /* Bad Block Count */
    unsigned short tblsz[8];     /* Block Allocation Size */
    unsigned short WritePoint[8];       /* Write Point */
    unsigned short ErasePoint[8];       /* Erase Point */
    unsigned short LimitBadBlock[8];   /* Bad Block Limit Value */

    unsigned char  ecctbl[256];    /* Parity Code Buffer */
    unsigned char  Ecc[6];        /* Ecc Code Buf */

    unsigned short BlockTable[8][1000];          /* Block Allocation Table */
    unsigned short WriteEntryTableList[8][1024];  /* Write Entry Table */
    unsigned short TableFlag[8];        /* Table Valid */
    unsigned char cbuf[SMA_SECTOR]; /* Common Data Buffer */
    unsigned char rdt[REDUD*32];    /* Redundant Data Buffer */
    unsigned char cpy_buf[SMA_SECTOR];  /* Copy Buffer - Data */
    unsigned char cpy_rdt[REDUD*32];    /* Copy Buffer - Redundant */
    unsigned char *MyMediaBuf;
    struct scatterlist *sg;             /* scatter gather */
    int           use_sg;               /* 0:no sg  other: # of gs_list */
    unsigned int  sg_no;                /* access no. of sg */
    unsigned int  sg_length;            /* size of sg buffer */
    unsigned char *sg_ptr;              /* access pointer of sg */
    unsigned char dvc[4];               /* Maker and Device Code */
    unsigned char cis_buf[SMA_SECTOR]; /* CIS Data Buffer */
    unsigned char cis_rdt[REDUD];       /* CIS Redundant Data Buffer */
    unsigned char hid_buf[SMA_SECTOR];  /* Hidden Data Buffer */
    unsigned char hid_rdt[REDUD];       /* Hidden Redundant Data Buffer */
    SMA03_FW_INFO  FWInfo;

/*       int     *lba_to_pba;    * logical to physical map */
/*       int     *pba_to_lba;    * physical to logical map */
};

#endif
